//
//  TRMovieInspector(RaisedEditing).m
//  Tryst
//
//  Created by Francisco Tolmasky on Fri Dec 26 2003.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRGeometry.h"
#import "TRMovieInspector.h"


@implementation TRMovieInspector (RaisedEditing)

// --- Raised Editing --- (Shadows around the selection)

// This routine takes a path, and returns an NSImage to use when drawing a raised editor look.
- (NSImage *)shadowImageAroundPath:(NSBezierPath*)aPath
{
    NSRect pathBounds= [aPath bounds];
    pathBounds.origin= NSZeroPoint;
    NSRect imageRect= NSMakeRect(0.0, 0.0, pathBounds.size.width+(2*kShadowBorder), pathBounds.size.height+(2*kShadowBorder));
    NSImage *shadowImage= nil;
    
    // 1. Create the image to return.
    shadowImage = [[NSImage alloc] initWithSize: imageRect.size];
    
    // We are going to be drawing this guy over and over at different sizes, so don't cache it with other images.
    [shadowImage setCachedSeparately: YES];
    
    // Make it more convenient to draw using compositeToPoint:
    [shadowImage setFlipped: YES];
    
    // 2. Lock focus on the image, so drawing goes into the image.
    [shadowImage lockFocus];
    
    // 3. Draw the shadow part
    [[NSView focusView] translateOriginToPoint: NSMakePoint(kShadowBorder, kShadowBorder)];
    
    if ([[self window] firstResponder]==self && [[self window] isKeyWindow])
    {
        // If the window is key, and we are first responder then draw a shadow.
        [NSGraphicsContext saveGraphicsState];
        
		NSShadow *shadow = [[NSShadow alloc] init];
        [shadow setShadowOffset: NSMakeSize(kShadowOffsetX, kShadowOffsetY)];
        [shadow setShadowBlurRadius: kShadowBlurRadius];

        [[NSColor whiteColor] set];
        [shadow set];

	// Fill.  The shadow will accumulate outside of the path.
	// ... and later, we'll leave transparency instead of 'whiteColor' by knocking a hole out of the middle.
        [aPath fill];
        [NSGraphicsContext restoreGraphicsState];
        [shadow release];
    }

    // 4. Knock a whole out of the middle, so that we are left with just shadow.
    [NSGraphicsContext saveGraphicsState];
    [aPath addClip];
    [[NSColor clearColor] set];
    NSRectFillUsingOperation([aPath bounds], NSCompositeCopy);
    [NSGraphicsContext restoreGraphicsState];

    // 5. Draw a nice border around the shadow.
    [[NSColor colorWithDeviceRed: kShadowBorderColor green: kShadowBorderColor blue: kShadowBorderColor alpha: 1.0f] set];
    [aPath stroke];
    [shadowImage unlockFocus];
    
    return [shadowImage autorelease];
}

- (void)drawRaisedEditorIfNecessaryInRect:(NSRect)clipRect
{
    // Get the range of glyphs we want to put the shadowed border around
    NSRange currentEditingGlyphRange = [self currentEditingGlyphRange];
    
    // If we found a glyph range, then draw a shadow around those glyphs...
    if(currentEditingGlyphRange.location!=NSNotFound)
    {
        // Compute 'editingGlyphRect' as the bounding rectangle for range of glyphs specified by currentEditingGlyphRange.
        BOOL requiresAddition= NO;
        NSTextContainer *textContainer= [self textContainer];
        NSLayoutManager *layoutManager= [textContainer layoutManager];
        NSRect editingGlyphRect= [layoutManager boundingRectForGlyphRange: currentEditingGlyphRange inTextContainer: textContainer];
        NSRect lastGlyphRect= [layoutManager boundingRectForGlyphRange: NSMakeRange(NSMaxRange(currentEditingGlyphRange)-1,1) inTextContainer: textContainer];
        // Give editingGlyphRect some padding, and floor the coordinates since the rect is used to create an image.
        if(requiresAddition= NSMaxX(lastGlyphRect)<NSMaxX(editingGlyphRect))
        {
            editingGlyphRect.size.height-= lastGlyphRect.size.height;
            lastGlyphRect.size.width= NSMaxX(lastGlyphRect)-kRightTabLocation;
        }
        if(editingGlyphRect.origin.x<(kRightTabLocation+5.0))
        {
            editingGlyphRect.origin.x= (kRightTabLocation+5.0);
            editingGlyphRect.size.width-= kRightTabLocation; //No +5.0 because already moved 5.0 before
        }
        else if(editingGlyphRect.origin.x>(kRightTabLocation+5.0))
        {
            editingGlyphRect.origin.x-= 1.0;
            editingGlyphRect.size.width+= 2.0;
        }
        editingGlyphRect= NSInsetRect(editingGlyphRect, -2.0, -2.0);
        lastGlyphRect= NSInsetRect(lastGlyphRect,-2.0,-2.0);
        // Now, generate the image with the shadow.
        NSBezierPath *shadowPath= [NSBezierPath bezierPath];
        [shadowPath moveToPoint: NSMakePoint(0.5,0.5)];
        [shadowPath lineToPoint: NSMakePoint(TRFixEdge(NSWidth(editingGlyphRect)),0.5)];
        [shadowPath lineToPoint: NSMakePoint(TRFixEdge(NSWidth(editingGlyphRect)),NSHeight(editingGlyphRect)-0.5)];
        [shadowPath lineToPoint: NSMakePoint(TRFixEdge(NSWidth(lastGlyphRect))-4.0,NSHeight(editingGlyphRect)-0.5)];
        if(requiresAddition) [shadowPath lineToPoint: NSMakePoint(TRFixEdge(NSWidth(lastGlyphRect))-4.0,NSHeight(editingGlyphRect)+NSHeight(lastGlyphRect)-4.5)];
        [shadowPath relativeLineToPoint: NSMakePoint(-TRFixEdge(NSWidth(lastGlyphRect))+4.5,0.0)];
        [shadowPath lineToPoint: NSMakePoint(0.5,0.5)];
        [shadowPath setLineCapStyle: NSSquareLineCapStyle];
        [shadowPath setLineJoinStyle: NSBevelLineJoinStyle];
        
        NSImage *shadowImage= [self shadowImageAroundPath: shadowPath];
        
        // Draw the image.  (Note, since we punched a whole out of the middle of the image we don't need to redraw the text)
        // NSLog(@"(%i,%i)",currentEditingGlyphRange.location,currentEditingGlyphRange.length);
        NSPoint compositePoint= NSMakePoint(NSMinX(editingGlyphRect)-kShadowBorder, NSMinY(editingGlyphRect)+[shadowImage size].height-kShadowBorder);
        if(requiresAddition)
        {
            [[NSColor whiteColor] set];
            float height= NSMinY(lastGlyphRect)-NSMinY(editingGlyphRect);
            NSRectFill(NSMakeRect(5.0,NSMaxY(lastGlyphRect)-2.0-height,kRightTabLocation,height));
        }
		
        [shadowImage compositeToPoint: compositePoint operation: NSCompositeSourceOver];
    }
}

@end
